@@ -1,4 +1,4 @@
-# $Id: AAA.pm 40646 2013-11-26 15:34:27Z anton $
+# $Id: AAA.pm 46953 2017-09-13 08:30:54Z anton $
 # $URL: https://svn.uvt.nl/its-id/trunk/sources/pwdmodifier/libpwdmodifier/lib/UvT/PwdModifier/AAA.pm $
 
 use strict;
@@ -7,9 +7,11 @@ use warnings FATAL => 'all';
 package UvT::PwdModifier::AAA;
 
 use Spiffy -Base;
+use Data::Dumper;
 
 field 'cf', -init => 'die "no cf configured\n"';
 field 'dir', -init => 'die "no directory specified\n"';
+#field 'corestaffFilter', - init => 'die "no protectCorestaffFilter specified\n"';
 
 sub verify_password {
 	return $self->dir->verify_password(@_);
@@ -26,13 +28,43 @@ sub verify_ipaddress {
 	return "address: '$ipaddress' outside of range ($adminipacl)";
 }
 
+
+
+sub isCorestaff {
+	my $dir = $self->dir;
+	my $victim = shift;
+	my $cf = $self->cf;
+	my $corestaffFilter = "|";
+	
+	my @corestaffFilterList = $cf->value('corestaffFilters');
+	foreach my $filterElement (@corestaffFilterList){
+		# verwijder leading spaces, anders worden die ook omgezet in \20
+		$filterElement =~ s/^\s+//;
+		$corestaffFilter .= $filterElement;
+	}
+	$corestaffFilter =~ s{\s}{\\20}g;
+	
+	my $filter = "(&(uid=$victim)($corestaffFilter))";
+#	warn "coreStaffFilter: $filter";
+	my $entries = $dir->getLdapEntriesByFilter($filter);
+	return @$entries[0];
+}
+
+	
 sub verify_authorization {
-	my ($admin, $victim, $privParam) = @_;
+	my ($admin, $victim, $privParam, $requestedAccess) = @_;
 	my $priv = 'pwd_change';  
 	$priv = $privParam if $privParam;
+	my $reqAcc = "readOnly";
+	$reqAcc = $requestedAccess if $requestedAccess;
+	
+	warn "verifying authorization, admin: $admin, victim: $victim, priv: $priv, requestedAccess: $reqAcc";
+	
+	if ($reqAcc eq "writeAccess" ) {
+		return "Explanation: No changes allowed for account: $victim."
+			if ($self->isCorestaff($victim));
+	}
 	
-	warn "verifying authorization, admin: $admin, victim: $victim, priv: $priv";
-
 	my $dir = $self->dir;
 
  	# 1) fetch de org.stat. van het victim en zijn uvt-lau attrs
@@ -41,8 +73,8 @@ sub verify_authorization {
 
 	my @orgstats = $entry->orgstats;
 	my @uvtlaus = $entry->laus;
-#	warn "orgstats: @orgstats";
-#	warn "uvtlaus: @uvtlaus";
+	#	warn "orgstats: @orgstats";
+	#	warn "uvtlaus: @uvtlaus";
 
 	# 2) stel een lijst samen
 	my %auths = ("$priv" => undef);
@@ -62,8 +94,9 @@ sub verify_authorization {
 	if($err) {
 		warn "admin: $admin is not authorized";
 		return "$err, $admin is not allowed to administrate account: $victim";
-	} else {
-		warn "admin: $admin is authorized to administrate account: $victim";
-		return;
 	}
+
+	warn "admin: $admin is authorized to administrate account: $victim";
+	return;
 }
+
