@@ -1,4 +1,4 @@
-# $Id: ControlTower.pm 43403 2015-06-04 08:03:06Z anton $
+# $Id: ControlTower.pm 46333 2017-03-03 16:31:35Z anton $
 # $URL: https://svn.uvt.nl/its-id/trunk/sources/pwdmodifier/libpwdmodifier/lib/UvT/PwdModifier/ControlTower.pm $
 use strict;
 package UvT::PwdModifier::ControlTower;
@@ -59,7 +59,7 @@ sub setPasswordStatus {
 	my $requestedPasswordStatus = $passwordStatus;
 
   commandSwitch: {
-	  $command =~ /^blockaccount|^disablePassword/i && do {$requestedPasswordStatus = 'closed'; last commandSwitch};
+	  $command =~ /^blockaccount|^disablePassword|invalidatePassword/i && do {$requestedPasswordStatus = 'closed'; last commandSwitch};
 	  $command =~ /^settmppwd/i && do {$requestedPasswordStatus = 'hijacked'; last commandSwitch};
 	  $command =~ /^unblockaccount|^unsettmppwd/i && do {$requestedPasswordStatus = 'normal'; last commandSwitch};
 	  $command =~ /^setpwd|^syncpassword/i && do {$requestedPasswordStatus = 'normal'; last commandSwitch};
@@ -87,12 +87,14 @@ sub regulateTraffic {
 	my $passwordStatus = $userinfo->{passwordStatus} // '';
 	my $passwordExpirationStage = $userinfo->{passwordExpirationStage} // '';
 	my $accountStatus = $userinfo->{accountStatus} // '';
-	my $accountBlocked = $userinfo->{accountBlocked};
-	my $accountTemporarilyDisabled = $userinfo->{accountTemporarilyDisabled};
-	my $unpaidFees = $userinfo->{unpaidFees};
+	my $accountBlocked = $userinfo->{accountBlocked} // '';
+	my $accountTemporarilyDisabled = $userinfo->{accountTemporarilyDisabled} // '';
+	my $unpaidFees = $userinfo->{unpaidFees} // '';
 #	my $passwordChangeCount = $userinfo->{passwordChangeCount} // '';
 	my $datePasswdChanged = $userinfo->{datePasswdChanged} // '';
-
+	my $passwordInvalidated = $userinfo->{passwordInvalidated} // '';
+	my $authenticationMethod = $userinfo->{authenticationMethod} // '';
+	
 	my $uid = $userinfo->{uid};
 
 	my $admin = 0; 
@@ -148,7 +150,7 @@ sub regulateTraffic {
   statusSwitch: {
 
 	  last statusSwitch
-				  if $command eq 'getadinfo';
+		  if $command eq 'getadinfo';
 
 	  $passwordStatus =~ /^closed/i 
 		  && do {
@@ -174,6 +176,19 @@ sub regulateTraffic {
 			  if ($unpaidFees){
 				  push(@denied, "\$unpaidFees_txt. $unpaidFees");
 				  push(@denied, '$contactStudentDesk_txt');
+				  last statusSwitch;
+			  }
+			  
+			  if ($passwordExpirationStage =~ /admin\-required/i){ 
+				  push(@denied, '$adminRequired_txt') unless $admin;
+				  last statusSwitch;
+			  }
+
+			  if ($passwordInvalidated){
+				  if ( $command =~ /setpwd/ && $authenticationMethod ne 'token'){
+					  push(@denied, '$passwordInvalidated_txt') unless $admin;
+				  }			
+				  last statusSwitch;
 			  }
 			  last statusSwitch;
 	  };
@@ -185,13 +200,6 @@ sub regulateTraffic {
 			  last statusSwitch;
 	  };
 
-	  $passwordStatus =~ /admin\-required/i 
-		  && do { 
-			  last statusSwitch if $command =~ /qamstatus/i;
-			  push(@denied, '$adminRequired_txt') unless $admin;
-			  last statusSwitch;
-	  };
-
 	  $passwordStatus =~ /^normal/i 
 		  && do { 
 			  $self->needlessRequest($uid, $passwordStatus, $command) if $command =~ /^unblockAccount|^unsetTmpPwd/i; 
