@@ -1,4 +1,4 @@
-# $Id: Directory.pm 45637 2016-09-05 14:08:10Z anton $
+# $Id: Directory.pm 46950 2017-09-12 15:20:37Z anton $
 # $URL: https://svn.uvt.nl/its-id/trunk/sources/pwdmodifier/libpwdmodifier/lib/UvT/PwdModifier/Directory.pm $
 
 use strict;
@@ -74,6 +74,7 @@ sub accountTemporarilyDisabled {
 
 
 sub stuff {
+	my $flattenArrays = shift;
 	my $entry = $self->ldapEntry;
 
 	my %userinfo;
@@ -97,6 +98,7 @@ sub stuff {
         passwordExpirationLastReminder
         passwordExpirationExpired
         passwordExpirationAdminRequired
+		passwordInvalidated
         passwordChangeCount
         passwordChangeDate
         passwordStatus
@@ -126,6 +128,10 @@ sub stuff {
     $userinfo{studentaccount} = $uid =~ /^(s\d{6}|u\d{7})$/i;
 
 	while (my ($key, $value) = each (%userinfo)){
+		if (defined($flattenArrays)) {
+			$value = join(', ', @{$value}) if ref($value) eq 'ARRAY';
+			$value = '' unless $value;
+		}
 		utf8::decode($value) or utf8::upgrade($value);
 		$userinfo{$key} = $value;
 	}
@@ -184,7 +190,7 @@ sub ldap_connection {
 		my $state;
 		my $debug = $self->debug;
 		warn "debug: $debug" if $debug;
-		warn "Connecting to ldapserver: $server" if $self->debug;
+		warn "Connecting to ldapserver: $server"; # if $self->debug;
 		$ldap = new Net::LDAP($server,
 							  # onerror is o.a. NODIG om te reageren op foutieve authenticatie         
 							  # nadeel is gvd dat je niet meer kunt zien waar je error vandaan komt :(
@@ -455,7 +461,7 @@ sub getRcryptPassword {
 }
 
 sub toggleAccountState {
-	my ($attrib, $uid, $on, $label, $by) = @_;
+	my ($attrib, $uid, $on, $by) = @_;
 	# ignore label
 	my ($entry, $ldap) = $self->getEntryByUID($uid);
 
@@ -465,7 +471,8 @@ sub toggleAccountState {
 		my $content = sprintf("$year-%02d-%02d %02d:%02d $by",$mon, $mday, $hour, $min);
 		$entry->replace($attrib, $content);
 	} else {
-		$entry->delete($attrib);
+		# throws an exception it the attrib does not exist \o/
+		$entry->delete($attrib) if $entry->get_value($attrib);
 	}
 
 	my $msg = $entry->update($ldap);
@@ -511,7 +518,7 @@ sub resetPasswordExpirationStatus {
 }
 
 sub removeEnclosingBrackets {
-	my $_ = shift;
+	local $_ = shift;
 
 	while (/^\(/ and /\)$/){
 		s/^\(//;
@@ -520,4 +527,51 @@ sub removeEnclosingBrackets {
 	$_;
 }
 
+sub ldapEncodeFilter {
+	#
+	# GEPIKT van ldapbrowser:Ldapsearch.pm:ldapsearchencode
+	#
+	#   http://www.cse.ohio-state.edu/cgi-bin/rfc/rfc2254.html
+	#
+	#   If a value should contain any of the following characters
+	#
+	#             Character       ASCII value
+	#             ---------------------------
+	#             *               0x2a
+	#             (               0x28
+	#             )               0x29
+	#             \               0x5c
+	#             NUL             0x00
+	#
+	#     the character must be encoded as the backslash '\' character (ASCII
+	#     0x5c) followed by the two hexadecimal digits representing the ASCII
+	#     value of the encoded character.
+    # comfort emacs ;(
+	#  map {$specialchars->{$_}=1 } split (//,'\()\*');
+
+	# Het simpelst is gewoon ALLES coderen, maar dat vinden mensenogen niet fijn.
+	# .. who cares?..
+	#  my $encoded='\\'.join('\\', map {sprintf ("%x",$_)} unpack('U*',$str));
+
+	#
+	# alternatief: codeer alleen de noneword characters,
+	# NB. blijf van de wildcard '*' af!
+	#
+    my $str = shift;
+	my $res = '';
+	if ($str) {
+		foreach my $char (split (//,$str)) {
+			if ($char =~ /\W/ and $char !~ /\*/) {
+				$res .= '\\' . sprintf("%x",ord($char));
+			} else {
+				$res .= $char;
+			}
+		}
+	}
+	$res;
+}
+
+
+
+
 0;
